/*
 * Extra Windows Position Manager for Gaim (Mostly MS Windows)
 *
 * Gaim is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <windows.h>
#include <assert.h>

#include "extpos.h"

static LRESULT CALLBACK shell_proc(int code, WPARAM wp, LPARAM lp)
{
    if(code == HSHELL_WINDOWCREATED)
    {
        if(g_extpos_data.hwnd_created)
            g_extpos_data.hwnd_created((HWND)wp);
    }

    if(g_extpos_data.shell_hook)
        CallNextHookEx(g_extpos_data.shell_hook, code, wp, lp);
    return 0;
}

static LRESULT CALLBACK cbt_proc(int code, WPARAM wp, LPARAM lp)
{
    if(code == HCBT_MOVESIZE)
    {
        if(g_extpos_data.hwnd_movesized)
            g_extpos_data.hwnd_movesized((HWND)wp, (RECT *)lp);
    }

    if(g_extpos_data.cbt_hook)
        CallNextHookEx(g_extpos_data.cbt_hook, code, wp, lp);
    return 0;
}

int hook_init(extpos_data_t *extpos_data,
    CALLBACK_HWND_CREATED *hwnd_created,
    CALLBACK_HWND_MOVESIZED *hwnd_movesized)
{
    DWORD thread_id = GetCurrentThreadId();
    HMODULE hmod = GetModuleHandle("extpos.dll");

    extpos_data->shell_hook = SetWindowsHookEx(WH_SHELL,
        (HOOKPROC)shell_proc, hmod, thread_id);
    extpos_data->cbt_hook = SetWindowsHookEx(WH_CBT,
        (HOOKPROC)cbt_proc, hmod, thread_id);

    extpos_data->hwnd_created = hwnd_created;
    extpos_data->hwnd_movesized = hwnd_movesized;

    return (extpos_data->shell_hook != NULL);
}

void hook_close(extpos_data_t *extpos_data)
{
    extpos_data->hwnd_created = NULL;
    extpos_data->hwnd_movesized = NULL;

    if(extpos_data->shell_hook)
    {
        UnhookWindowsHookEx(extpos_data->shell_hook);
        extpos_data->shell_hook = NULL;
    }

    if(extpos_data->cbt_hook)
    {
        UnhookWindowsHookEx(extpos_data->cbt_hook);
        extpos_data->cbt_hook = NULL;
    }
}
