/*
 * Extra Windows Position Manager for Gaim (Mostly MS Windows)
 *
 * Gaim is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <windows.h>
#include <assert.h>

#include "debug.h"

#include "extpos.h"

static LRESULT CALLBACK hotkey_wndproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
    if(msg == WM_HOTKEY)
    {
        extpos_data_t *extpos_data;
        HOTKEY_ID hotkey_id = (HOTKEY_ID)(wp & 0x0fff);

        extpos_data = (extpos_data_t *)GetWindowLongPtr(hwnd, GWLP_USERDATA);
        assert(extpos_data && extpos_data->is_loaded);

        if(extpos_data && (hotkey_id < IDHK_Max))
        {
            HOTKEY_DATA *hotkey_data = &extpos_data->hotkey_data[wp & 0x0fff];

            if(hotkey_data->callback)
                hotkey_data->callback(extpos_data);
        }
    }

    return DefWindowProc(hwnd, msg, wp, lp);
}

static void init_extpos_hotkey_window(extpos_data_t *extpos_data)
{
    if(!extpos_data->hotkey_hwnd)
    {
        WNDCLASSEX wc = {0};

        extpos_data->last_conv_list = NULL;

        wc.cbSize = sizeof(wc);
        wc.lpfnWndProc = hotkey_wndproc;
        wc.hInstance = GetModuleHandle("extpos.dll");
        wc.lpszClassName = "extpos_hotkey_cb";
        RegisterClassEx(&wc);

        // Create a dummy window that listens for the hotkey
        extpos_data->hotkey_hwnd = CreateWindow("extpos_hotkey_cb",
            NULL, 0, 0, 0, 0, 0, HWND_MESSAGE, NULL,
            GetModuleHandle("extpos.dll"), NULL);
        if(!extpos_data->hotkey_hwnd)
        {
            purple_debug_error("extpos", "CreateWindow failed.");
        }
        else
        {
            SetWindowLongPtr(extpos_data->hotkey_hwnd, GWLP_USERDATA,
                (LONG_PTR)extpos_data);
        }
    }
}

void extpos_register_hotkey(extpos_data_t *extpos_data,
    HOTKEY_ID hotkey_id, int mods, int vk,
    CALLBACK_HOTKEY *hotkey_callback)
{
    HOTKEY_DATA *hotkey_data = &extpos_data->hotkey_data[hotkey_id];

    if(hotkey_data->callback)
    {
        UnregisterHotKey(extpos_data->hotkey_hwnd, hotkey_data->vk);
        hotkey_data->callback = NULL;
    }

    if(hotkey_callback && mods != -1 && vk != -1)
    {
        init_extpos_hotkey_window(extpos_data);

        if(RegisterHotKey(extpos_data->hotkey_hwnd,
            (0xb000 | hotkey_id), mods, vk))
        {
            hotkey_data->mods = mods;
            hotkey_data->vk = vk;
            hotkey_data->callback = hotkey_callback;
        }
        else
        {
            purple_debug_error("extpos", "RegisterHotKey(%d,%d) failed: %d",
                mods, vk, GetLastError());
        }
    }
}

void extpos_close_hotkeys(extpos_data_t *extpos_data)
{
    if(extpos_data->hotkey_hwnd)
    {
        int i;

        for(i = 0; i < IDHK_Max; i++)
        {
            extpos_register_hotkey(extpos_data, (HOTKEY_ID)i, 0, 0, NULL);
        }

        DestroyWindow(extpos_data->hotkey_hwnd);
        extpos_data->hotkey_hwnd = NULL;
    }
}
